function main(varargin)
    % main.m - main graphical user interface for cell counting tool
    %
    % Copyright (C) 2011-2012 Kristian Bredies (kristian.bredies@uni-graz.at)
    %
    %   This program is free software; you can redistribute it and/or modify
    %   it under the terms of the GNU General Public License as published by
    %   the Free Software Foundation; either version 2, or (at your option)
    %   any later version.
    %
    %   This program is distributed in the hope that it will be useful,
    %   but WITHOUT ANY WARRANTY; without even the implied warranty of
    %   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    %   GNU General Public License for more details.
    %
    %   You should have received a copy of the GNU General Public License
    %   along with this program; if not, write to the Free Software
    %   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA
    %   02110-1301, USA.
    
    % initialize
    param.action = 'default';
    param.files = {};
    param.debug = 1;
    param = process(param);
    
    % load GUI
    hwnd = openfig('main_ui.fig');
    figure(hwnd);
    gui = get_gui_handles();
    set(gui.status, 'String', {'Ready.'}); 
    initialize_callbacks();
    update_gui_elements();
    
    % check for call with 'quit'
    quit_on_close = 0;
    if (nargin == 1)
        if (strcmp(varargin{1}, 'quit'))
            quit_on_close = 1;
        end
    end

    %%%%%%%%%%%%%%%%%%%%
    % callback functions
    
    % segmentation
    function do_counting(obj, event)
        data = guidata(gcbo);
        
        if (isfield(data, 'exe'))
            % give signal to terminate
            mesg(1,'Sending termination request...');
            
            data.abort = 1;
            guidata(gcbo,data);
        else
            % perform action
            param.action = 'do';
        
            oldstr = get(gui.seg.do, 'String');
            set(gui.seg.do, 'String', 'Abort');
            data.exe = 1;
            guidata(gcbo, data);
            
            try
                param_discard = process(param);
            catch ME
                message = regexprep(ME.message, '<(.*?)>', ' ');
                
                mesg(1, sprintf('Exception (%s)', ME.identifier));
                mesg(1, message);
                
                stack = 'Stack: ';
                for i=1:length(ME.stack)
                    stack = strcat(stack, sprintf('%d@%s; ', ...
                        ME.stack(i).line, ME.stack(i).name));
                end
                mesg(1, stack);
            end  
            
            data = guidata(gcbo);
            data = rmfield(data,'exe');
            guidata(gcbo, data);
            set(gui.seg.do, 'String', oldstr);
        end
    end

    function counting_conf(obj, event)
        param.action = 'configure';
        
        param = process(param);
    end

    % report
    function generate_report(obj, event)
        param.action = 'generate';
        param = report(param);
    end
    
    % menu routines
    function menu_load(obj, event)
        [name, path] = uigetfile('*.mat','Load job');
        if (ischar(name))
            contents = load([path name], 'param');
            param = contents.param;
            
            % add path of the job file if necessary
            if (isfield(param, 'files'))
                for i=1:length(param.files)
                    [pathstr, name, ext] = fileparts(param.files{i});
                    if (isempty(pathstr))
                        param.files{i} = [path name ext];
                    end
                end
            end
            
            update_gui_elements();
        end
    end
    
    function menu_save(obj, event)
        [name, path] = uiputfile('*.mat','Save job as');
        if (ischar(name))
            save([path name], 'param');
        end
    end
    
    function menu_quit(obj, event)
        if (strcmp(questdlg('Really quit?'),'Yes'))
            delete(gcbf);
            if (quit_on_close)
                exit;
            end
        end
    end

    function menu_about(obj, event)
        hdlg = openfig('about_ui.fig');
        waitfor(hdlg);
    end

    function menu_help(obj, event)
        web('doc/short_instructions.html');
    end

    % file selection stuff
    function files_add(obj, event) 
        [name, path] = uigetfile({'*.bmp;*.gif;*.jpg;*.jpeg;*.png;*.tif;*.tiff;*.mat', ...
            'All image formats'; ...
            '*.bmp', 'Windows bitmap (*.bmp)'; ...
            '*.gif', 'Graphics interchange format (*.gif)'; ...
            '*.jpg;*.jpeg', 'Joint photographic experts group (*.jpg, *.jpeg)'; ...
            '*.png', 'Portable network graphics (*.png)'; ...
            '*.tif;*.tiff', 'Tagged image file format (*.tif, *.tiff)'; ...
            '*.mat', 'AMIRA Matlab export format (*.mat)'}, ...
            'Select image files', 'Multiselect', 'on');
        
        if (ischar(name) || iscell(name))
            % get index to insert
            firstindex = get(gui.files, 'Value');
            if (isempty(firstindex))
                firstindex = 1;
            else
                firstindex = min(firstindex) + 1;
            end
            
            % deal with inconsistent uigetfile output
            if (iscell(name))
                for i=1:length(name)
                    name{i} = [path name{i}]; 
                end
            else
                name = {[path name]};
            end
            
            param.files = [param.files{1:firstindex-1} ...
                name param.files{firstindex:end}];
            selected = firstindex:(firstindex+length(name)-1);
            
            update_file_box(selected);            
        end
    end

    function files_del(obj, event)
        sel = get(gui.files,'Value');
        
        if (~isempty(sel))
            mask =  ~ismember(1:length(param.files), sel);
            param.files = param.files(mask);
            update_file_box([]);
        end
    end

    function files_move_up(obj, event)
        sel = sort(get(gui.files,'Value'));
        
        N = length(param.files);
        % insert dummy (0)
        perm = 0:N;
        % permute according to selection
        for i=1:length(sel)
            j = sel(i);
            if (j <= N)
                swap = perm(j+1);
                perm(j+1) = perm(j);
                perm(j) = swap;
            end
        end
        % remove dummy
        perm = perm(perm ~= 0);
        
        % compute inverse permutation
        invperm = zeros(N,1);
        invperm(perm) = 1:N;
        
        % apply permutation und update
        files = cell(N,1);
        for i = 1:N
            files{i} = param.files{perm(i)};
        end
        param.files = files;
        newsel = invperm(sel); 
        
        update_file_box(newsel);
    end

    function files_move_down(obj, event)
        sel = sort(get(gui.files,'Value'),2,'descend');
        
        N = length(param.files);
        % insert dummy (0)
        perm = [1:N 0];
        % permute according to selection
        for i=1:length(sel)
            j = sel(i);
            if (j <= N)
                swap = perm(j+1);
                perm(j+1) = perm(j);
                perm(j) = swap;
            end
        end
        % remove dummy
        perm = perm(perm ~= 0);
        
        % compute inverse permutation
        invperm = zeros(N,1);
        invperm(perm) = 1:N;
        
        % apply permutation und update
        files = cell(N,1);
        for i = 1:N
            files{i} = param.files{perm(i)};
        end
        param.files = files;
        newsel = invperm(sel); 
        
        update_file_box(newsel);
    end

    % replacement rule stuff
    function file_pat_changed(obj, event)
        str = get(gui.file.fluor_pat, 'String');
        param.seg.post_fluor_pat = str;
    end

    function file_rep_changed(obj, event)
        str = get(gui.file.fluor_rep, 'String');
        param.seg.post_fluor_rep = str;
    end

    function gen_img_changed(obj, event)
        val = get(gui.file.gen_img, 'Value');
        param.seg.post_gen_img = val;
    end
        
    %%%%%%%%%%%%%
    % gui stuff
    function update_file_box(sel)
        str = cell(length(param.files),1);
        for i=1:length(param.files)
            [path,name,ext] = fileparts(param.files{i});
            str{i} = [name ext];
        end
        set(gui.files, 'ListboxTop', min(max(1,length(param.files)), ...
            max(1, get(gui.files, 'ListboxTop'))));
        set(gui.files,'String',str);
        set(gui.files, 'Value', sel);        
    end

    function update_gui_elements()
        % sets the adjustable elements according to param

        update_file_box([]);
        set(gui.files, 'Value', []);
        
        set(gui.file.fluor_pat, 'String', param.seg.post_fluor_pat);
        set(gui.file.fluor_rep, 'String', param.seg.post_fluor_rep);
        set(gui.file.gen_img, 'Value', param.seg.post_gen_img);
    end
    
    function initialize_callbacks()
        % set all callback functions
        set(hwnd, 'CloseRequestFcn', @menu_quit);
        
        % menu 
        set(gui.menu.load, 'Callback', @menu_load);
        set(gui.menu.save, 'Callback', @menu_save);
        set(gui.menu.quit, 'Callback', @menu_quit);
        set(gui.menu.about, 'Callback', @menu_about);
        set(gui.menu.help, 'Callback', @menu_help);
        
        % files selection
        set(gui.file.add, 'Callback', @files_add);
        set(gui.file.del, 'Callback', @files_del);
        set(gui.file.up, 'Callback', @files_move_up);
        set(gui.file.down, 'Callback', @files_move_down);
        
        % replacement rule
        set(gui.file.fluor_pat, 'Callback', @file_pat_changed);
        set(gui.file.fluor_rep, 'Callback', @file_rep_changed);
        set(gui.file.gen_img, 'Callback', @gen_img_changed);
        
        % segmentation buttons
        set(gui.seg.do, 'Callback', @do_counting);
        set(gui.seg.conf, 'Callback', @counting_conf);
        
        % report buttons
        set(gui.report.do, 'Callback', @generate_report);
    end
    
    function gui = get_gui_handles()
        % output elements
        gui.axes = findobj(hwnd,'Tag','outputwindow');
        gui.status = findobj(hwnd,'Tag','outputtext');
        
        % menu
        gui.menu.load = findobj(hwnd,'Tag','load');
        gui.menu.save = findobj(hwnd,'Tag','save');
        gui.menu.quit = findobj(hwnd,'Tag','quit');
        gui.menu.help = findobj(hwnd,'Tag','instructions');
        gui.menu.about = findobj(hwnd,'Tag','about');
        
        % file dialog
        gui.files = findobj(hwnd,'Tag','files');
        gui.file.add = findobj(hwnd,'Tag','files_add');
        gui.file.del = findobj(hwnd,'Tag','files_remove');
        gui.file.up = findobj(hwnd,'Tag','files_move_up');
        gui.file.down = findobj(hwnd,'Tag','files_move_down');
        gui.file.fluor_pat = findobj(hwnd,'Tag','fluor_pat');
        gui.file.fluor_rep = findobj(hwnd,'Tag','fluor_rep');
        gui.file.gen_img = findobj(hwnd,'Tag','generate_seg_image');
        
        % segmentation dialog
        gui.seg.do = findobj(hwnd,'Tag','counting_do');
        gui.seg.conf = findobj(hwnd,'Tag','counting_configure');
        
        % report dialog
        gui.report.do = findobj(hwnd,'Tag','report_do');
        gui.report.expl = findobj(hwnd,'Tag','report_explore');        
    end
end
