function param = report(param)
    % report generation module
    % ------------------------
    %
    % - generates basic reports
    % - provides a data exploration gui
    % 
    % param: parameter structure - fields:  
    % 
    % action - 'generate' generates report after asking user
    %          'explore' invokes data exploration dialog
    %
    % files - list of images to process
    %
    % debug - if set, debug output will be produced
    %
    %
    % Copyright (C) 2011-2012 Kristian Bredies (kristian.bredies@uni-graz.at)
    %
    %   This program is free software; you can redistribute it and/or modify
    %   it under the terms of the GNU General Public License as published by
    %   the Free Software Foundation; either version 2, or (at your option)
    %   any later version.
    %
    %   This program is distributed in the hope that it will be useful,
    %   but WITHOUT ANY WARRANTY; without even the implied warranty of
    %   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    %   GNU General Public License for more details.
    %
    %   You should have received a copy of the GNU General Public License
    %   along with this program; if not, write to the Free Software
    %   Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA
    %   02110-1301, USA.
    
    
    % check for action
    if (~isfield(param,'action'))
        mesg(1, 'Could not determine what to do.');
        return;
    end
    
    isdebug = isfield(param,'debug');
    
    action = 0;
        
    if (strcmp(param.action,'generate'))
        % configuration
        action = 1;
        
        % check for file list
        if (~isfield(param,'files'))
            mesg(1, 'No files to process specified.');
            return;
        end
        
        numfiles = length(param.files);
        if (numfiles == 0)
            mesg(1, 'No files to process specified.');
            return;
        end
        
        % first ask for filename etc.
        mesg(isdebug, 'Invoking export dialog.');
        [param, okay] = configure_report(param);
        
        if (okay)
            mesg(isdebug, 'Generating report...');
            param = generate_report(param);
        else
            mesg(isdebug, 'Report generation canceled.');
        end
    end
    
    
    if (strcmp(param.action,'explore'))
        % batch processing
        action = 1;
        
        mesg(isdebug, 'Invoking data exploration dialog.');
        mesg(isdebug, 'Sorry, not implemented yet.');
    end
    
    if (action == 0)
        mesg(1, sprintf('Action ''%s'' not implemented.', param.action));
    end 
end

%%%%%%%%%%%%%%%%%%%%%%%
% report configuration
function [param, save_settings] = configure_report(param)
    param_old = param;

    hdlg = openfig('rep_gen_ui.fig');
    
    if (~isfield(param,'report'))
        % initialize with default values
        param.report.outputfile = 'report.csv';
    end
    
    gui = get_gui_handles();
    initialize_values();
    initialize_callbacks();
    
    save_settings = 0;
    waitfor(hdlg);
    if (~save_settings)
        param = param_old;
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%
    % callbacks
    function okay_button(obj, event)
        save_values();
        save_settings = 1;
        close(gcbf);
    end

    function cancel_button(obj, event)
        close(gcbf);
    end

    function filename_select(obj, event)
        [name, path] = uiputfile({'*.csv', ...
            'Comma separated values (*.csv)'}, 'Save report as');
        if (ischar(name))
            param.report.outputfile = [path name];
            set(gui.filename, 'String',param.report.outputfile);
        end
    end
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % initalization and save routines
    function initialize_callbacks()
        % pushbutton callbacks
        set(gui.okay, 'Callback', @okay_button);
        set(gui.cancel, 'Callback', @cancel_button); 
        
        % filename selection callback
        set(gui.filename_sel,'Callback', @filename_select);
    end
    
    function save_values()
        param.report.outputfile = get(gui.filename,'String');        
    end
    
    function initialize_values()
        set(gui.filename, 'String', param.report.outputfile);
    end
    
    function gui = get_gui_handles()        
        gui.okay = findobj(hdlg,'Tag','okay');
        gui.cancel = findobj(hdlg,'Tag','cancel');
        
        gui.filename = findobj(hdlg,'Tag','filename');
        gui.filename_sel = findobj(hdlg,'Tag','filename_select');
    end
end

%%%%%%%%%%%%%%%%%%%%%
% report generation
function param = generate_report(param)
% generates report according to param.report
%
% valid param.report.types:
% csv_aggegrated - aggegrated list
% csv_detailed - detailed list


isdebug = isfield(param, 'debug');
file = fopen(param.report.outputfile, 'w');
fprintf(file, 'm/c/f=%d/%f/%d\n', param.seg.pre_medfilt_len, param.seg.pre_seg_thres, param.seg.pre_feat_size);
            
if (param.seg.pre_tv)
                fprintf(file, 'alpha/N=%f/%d\n', param.seg.pre_tv_reg, param.seg.pre_tv_iter);
            end


fprintf(file, 'file, cellnr, x, y\n')

N = length(param.files);
for i=1:N
    % get results filename
    curfile = param.files{i};
    [pathstr, name, ext] = fileparts(curfile);
    resfile = fullfile(pathstr, [name '_res.mat']);
    
    % process segmented file
    if (exist(resfile, 'file'))
        data = load(resfile,'seg');
        data = data.seg;
found_cell_number= double(length(union(data(data ~= 0),[])));
[width_data height_data] = size(data);
    mean_density = double(100* found_cell_number / ( width_data * height_data));
fprintf(file, '[%s%s:_Found_cells: %d]\n', name, ext, found_cell_number);
fprintf(file, '[%s%s:_Mean_density_per_10_x_10_px: %f]\n', name, ext, mean_density);
        mesg(isdebug, sprintf('Segmented data ''%s'' loaded.', resfile));
        
        %labels = union(data(data ~= 0), []);
        cellval = unique(data(data ~= 0));
        [X,Y] = meshgrid(1:size(data,2), 1:size(data,1));
        for n=1:length(cellval)
            massx = mean(X(data == cellval(n)));
            massy = mean(Y(data == cellval(n)));
            str = sprintf('%s%s,%d,%.2f,%.2f', name, ext, n, massx, massy);
            fprintf(file, [str '\n']);
        end
    else
        mesg(1,sprintf('Segmented data ''%s'' does not exist. Skipping.', resfile));
    end
    
end

fclose(file);


end